/**
 * Stores components.
 *
 * Key is the component id.
 * Value is the instantiated component.
 */
export const components = {};

/**
 * Stores lifecycle events to fire.
 *
 * Key is the event name.
 * Value is an array of callback functions.
 */
export const lifecycleEvents = {};
