class UnicornCacheError(Exception):
    pass


class UnicornViewError(Exception):
    pass


class ComponentLoadError(Exception):
    pass


class RenderNotModified(Exception):
    pass


class MissingComponentElement(Exception):
    pass


class MissingComponentViewElement(Exception):
    pass


class ComponentNotValid(Exception):
    pass
